import factory
import factory.django
from pathlib import Path

from django.core.files.uploadedfile import SimpleUploadedFile
from doma.models import Document, DocumentType

DOCUMENT_BASE_PATH = Path(__file__).parent
DOCUMENT_PATH = DOCUMENT_BASE_PATH / "document.pdf"
DOCUMENT_2_PATH = DOCUMENT_BASE_PATH / "document_2.pdf"


def get_test_document(document_path=DOCUMENT_PATH, sup=False):
    with open(document_path, "rb") as infile:
        if sup:
            return SimpleUploadedFile(document_path.name, infile.read())
        else:
            return infile.read()


class DocumentTypeFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = DocumentType

    name = factory.Sequence(lambda n: "Type %03d" % n)


class DocumentFactory(factory.django.DjangoModelFactory):
    class Meta:
        model = Document

    name = factory.Sequence(lambda n: "Document %03d" % n)
    type = factory.SubFactory(DocumentTypeFactory)
    file = factory.django.FileField(
        filename=DOCUMENT_PATH.name, data=get_test_document()
    )



import tempfile

USE_TZ = False
SECRET_KEY = "fake_key_for_testing"
INSTALLED_APPS = [
    "doma",
    "doma.tests",
]
MEDIA_ROOT = tempfile.mkdtemp(prefix="django-doma_test")
DATABASES = dict(
    default=dict(
        ENGINE="django.db.backends.sqlite3",
        NAME=":memory:",
    )
)



from django.core.exceptions import ValidationError
from django.db.models import ProtectedError
from django.test import TestCase
from freezegun import freeze_time

from doma.models import Document
from doma.tests.factories import (
    DocumentFactory,
    DocumentTypeFactory,
    get_test_document,
    DOCUMENT_2_PATH,
)


class DomaTestCase(TestCase):
    def setUp(self):
        self.t = DocumentTypeFactory()

    def test_document_delete_protect(self):
        """Tests if the PROTECT_AFTER switch works correctly."""
        with freeze_time("2021-01-01"):
            t_old = DocumentFactory()
        self.assertRaises(ProtectedError, t_old.delete)

    def test_preview_image(self):
        """Test if a preview image is set for the document."""
        d = DocumentFactory()
        self.assertIsNotNone(d.preview)

    def test_document_delete(self):
        """Tests if the PROTECT_AFTER switch works correctly."""
        t_new = DocumentFactory()
        t_new.delete()

    def test_file_change(self):
        d = DocumentFactory()
        d.file = get_test_document(sup=True)
        self.assertRaises(ValidationError, d.save)

    def test_document_replace_file(self):
        d = DocumentFactory()
        d2 = Document.objects.replace(d, get_test_document(DOCUMENT_2_PATH, sup=True))
        self.assertEqual(d2.replaces, d)
        print(d2.file.name, d.file.name)
